"""! @file filters.py
@brief Filter library for the application
@author xlanro00
"""

import numpy as np
#import matplotlib.pyplot as plt
import cv2 as cv


# Parent class for all the filters
class filter:
    def __init__(self, input_file, output_file, img):
        self.input_file = input_file
        self.output_file = output_file
        self.img = img


class average(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.ones((3, 3), np.float32) / 9
        self.img = cv.filter2D(self.img, -1, kernel)


class blur(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.blur(self.img, (3, 3))


class gaussian(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.GaussianBlur(self.img, (3, 3), 0)


class median(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.medianBlur(self.img, 1)


class bilateral(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        self.img = cv.bilateralFilter(self.img, 1, 75, 75)


class denoise(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        pass
        #does not work
        #img = cv.cvtColor(self.img, cv.COLOR_GRAY2BGR)
        #self.img = cv.fastNlMeansDenoisingColored(img, h=3)


class sharpen(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.array([[0, -1, 0], [-1, 5, -1], [0, -1, 0]])
        self.img = cv.filter2D(self.img, ddepth=-1, kernel=kernel)


class erode(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.ones((3, 3), np.uint8)
        self.img = cv.erode(self.img, kernel, 1)


class dilate(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = np.ones((3, 3), np.uint8)
        self.img = cv.dilate(self.img, kernel, 1)


# Erosion, then dilatation
class opening(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = cv.getStructuringElement(cv.MORPH_RECT, (3, 3))
        self.img = cv.morphologyEx(self.img, cv.MORPH_CLOSE, kernel)


# Dilatation, then erosion
class closing(filter):
    def __init__(self, input_file, output_file, img):
        super().__init__(input_file, output_file, img)

    def apply(self):
        kernel = cv.getStructuringElement(cv.MORPH_RECT, (3, 3))
        self.img = cv.morphologyEx(self.img, cv.MORPH_OPEN, kernel)
